@@ -43,7 +43,7 @@
   else beta.size <- ncol(trend)
   n.sim <- floor(n.iter/thin)
   ## remeber this rather odd coding for telling that S.start is from the prior !!!
-  if(!any(mcmc.input$S.start=="random")) Sdata <- as.double(as.vector(c(rep(0, n.sim*n - 1),1)))
+  if(any(mcmc.input$S.start=="random")) Sdata <- as.double(as.vector(c(rep(0, n.sim*n - 1),1)))
   else Sdata <- as.double(as.vector(c(S, rep(0, (n.sim - 1) * n))))
   phi.sample <- as.double(rep(phi, n.sim))
   acc.rate <- rep(0,floor(n.iter/1000)+1)
@@ -127,7 +127,7 @@
   if(is.null(ttvbetatt)) ttvbetatt <- matrix(0,beta.size,beta.size)
   n.sim <- floor(n.iter/thin)
   ## remeber this rather odd coding for telling that S.start is from the prior !!!
-  if(!any(mcmc.input$S.start=="random")) Sdata <- as.double(as.vector(c(rep(0, n.sim*n - 1),1)))
+  if(any(mcmc.input$S.start=="random")) Sdata <- as.double(as.vector(c(rep(0, n.sim*n - 1),1)))
   else Sdata <- as.double(as.vector(c(S, rep(0, (n.sim - 1) * n))))
   phi.sample <- as.double(rep(phi, n.sim))
   acc.rate <- rep(0,floor(n.iter/1000)+1)
@@ -196,71 +196,15 @@
   ##
   ## reading input
   ##
-  if(missing(model))
-    model <- model.glm.control()
-  else{
-    if(is.null(class(model)) || class(model) != "model.geoRglm"){
-      if(!is.list(model))
-        stop("binom.krige.bayes: the argument model only takes a list or an output of the function model.glm.control")
-      else{
-        model.names <- c("trend.d", "trend.l", "cov.model", "kappa", "aniso.pars", "lambda") 
-        model.user <- model
-        model <- list()
-        if(length(model.user) > 0){
-          for(i in 1:length(model.user)){
-            n.match <- match.arg(names(model.user)[i], model.names)
-            model[[n.match]] <- model.user[[i]]
-          }
-        }   
-        if(is.null(model$trend.d)) model$trend.d <- "cte"  
-        if(is.null(model$trend.l)) model$trend.l <- "cte"  
-        if(is.null(model$cov.model)) model$cov.model <- "matern"  
-        if(is.null(model$kappa)) model$kappa <- 0.5
-        model <- model.glm.control(trend.d = model$trend.d,
-                                   trend.l = model$trend.l,
-                                   cov.model = model$cov.model,
-                                   kappa = model$kappa,
-                                   aniso.pars = model$aniso.pars,
-                                   lambda = model$lambda)
-      }
-    }
-  }
+  if(missing(model)) model <- model.glm.control()
+  else model <- model.glm.check.aux(model, fct = "binom.krige.bayes")
   cov.model <- model$cov.model
   kappa <- model$kappa
   tausq.rel <- prior$tausq.rel
   ## reading prior input
   ##
-  if(missing(prior))
-    stop("binom.krige.bayes: argument prior must be given ")
-  else{
-    if(is.null(class(prior)) || class(prior) != "prior.geoRglm"){
-      if(!is.list(prior))
-        stop("binom.krige.bayes: argument prior only takes a list or an output of the function prior.glm.control")
-      else{
-        prior.names <- c("beta.prior", "beta", "beta.var.std", "sigmasq.prior",
-                         "sigmasq", "df.sigmasq", "phi.prior", "phi", "phi.discrete", "tausq.rel") 
-        prior.user <- prior
-        prior <- list()
-        if(length(prior.user) > 0){
-          for(i in 1:length(prior.user)){
-            n.match <- match.arg(names(prior.user)[i], prior.names)
-            prior[[n.match]] <- prior.user[[i]]
-          }
-        } 
-        if(is.null(prior$beta.prior)) prior$beta.prior <- "flat"
-        if(is.null(prior$sigmasq.prior)) prior$sigmasq.prior <- "uniform"
-        if(is.null(prior$phi.prior)) prior$phi.prior <- "uniform"
-        if(is.null(prior$tausq.rel)) prior$tausq.rel <- 0
-        prior <- prior.glm.control(beta.prior = prior$beta.prior,
-                                   beta = prior$beta, beta.var.std = prior$beta.var.std,
-                                   sigmasq.prior = prior$sigmasq.prior,
-                                   sigmasq = prior$sigmasq,  df.sigmasq = prior$df.sigmasq,
-                                   phi.prior = prior$phi.prior,
-                                   phi = prior$phi, phi.discrete = prior$phi.discrete, 
-                                   tausq.rel = prior$tausq.rel)
-      }
-    }
-  }
+  if(missing(prior))  stop("binom.krige.bayes: argument prior must be given ")
+  else prior <- prior.glm.check.aux(prior, fct = "binom.krige.bayes")
   beta.prior <- prior$beta.prior
   beta <- prior$beta
   beta.var <- prior$beta.var.std
@@ -272,42 +216,13 @@
   }
   phi.prior <- prior$phi.prior 
   phi <- prior$phi
-  if(sigmasq.prior == "fixed" & phi.prior != "fixed") stop("option for fixed sigmasq and random phi not implemented")
   if(phi.prior != "fixed") phi.discrete <- prior$phi.discrete
   else phi.discrete <- phi
   ##
   ## reading output options
   ##
-  if(missing(output))
-    output <- output.glm.control()
-  else{
-    if(is.null(class(output)) || class(output) != "output.geoRglm"){
-      if(!is.list(output))
-        stop("binom.krige.bayes: the argument output only takes a list or an output of the function output.glm.control")
-      else{
-        output.names <- c("sim.posterior","sim.predict", "keep.mcmc.sim","quantile","threshold","inference","messages.screen")      
-        output.user <- output
-        output <- list()
-        if(length(output.user) > 0){
-          for(i in 1:length(output.user)){
-            n.match <- match.arg(names(output.user)[i], output.names)
-            output[[n.match]] <- output.user[[i]]
-          }
-        }
-        if(is.null(output$sim.posterior)) output$sim.posterior <- TRUE
-        if(is.null(output$sim.predict)) output$sim.predict <- FALSE
-        if(is.null(output$keep.mcmc.sim)) output$keep.mcmc.sim <- TRUE  
-        if(is.null(output$quantile)) output$quantile <- TRUE 
-        if(is.null(output$inference)) output$inference <- TRUE
-        if(is.null(output$messages.screen)) output$messages.screen <- TRUE        
-        output <- output.glm.control(sim.posterior = output$sim.posterior,
-                                     sim.predict = output$sim.predict,
-                                     keep.mcmc.sim = output$keep.mcmc.sim, quantile = output$quantile,
-                                     threshold = output$threshold, inference = output$inference,
-                                     messages.screen = output$messages.screen)
-      }
-    }
-  }
+  if(missing(output)) output <- output.glm.control()
+  else output <- output.glm.check.aux(output, fct = "binom.krige.bayes")
   quantile.estimator <- output$quantile.estimator
   probability.estimator <- output$probability.estimator
   inference <- output$inference
@@ -385,32 +300,7 @@
   ## ##### preparing for MCMC -------------------------------------------------------
   ##
   if(missing(mcmc.input)) stop("binom.krige.bayes: argument mcmc.input must be given")
-  else{
-    if(is.null(class(mcmc.input)) || class(mcmc.input) != "mcmc.geoRglm"){
-      if(!is.list(mcmc.input))
-        stop("binom.krige.bayes: the argument mcmc.input only takes a list or an output of the function mcmc.control")
-      else{
-        mcmc.input.names <- c("S.scale", "Htrunc", "S.start", "burn.in", "thin", "n.iter", "phi.start",  "phi.scale")    
-        mcmc.input.user <- mcmc.input
-        mcmc.input <- list()
-        if(length(mcmc.input.user) > 0){
-          for(i in 1:length(mcmc.input.user)){
-            n.match <- match.arg(names(mcmc.input.user)[i], mcmc.input.names)
-            mcmc.input[[n.match]] <- mcmc.input.user[[i]]
-          }
-        }
-        if(is.null(mcmc.input$Htrunc)) mcmc.input$Htrunc <- "default"
-        if(is.null(mcmc.input$S.start)) mcmc.input$S.start <- "default"
-        if(is.null(mcmc.input$burn.in)) mcmc.input$burn.in <- 0
-        if(is.null(mcmc.input$thin)) mcmc.input$thin <- 10
-        if(is.null(mcmc.input$n.iter)) mcmc.input$n.iter <- 1000*mcmc.input$thin
-        if(is.null(mcmc.input$phi.start)) mcmc.input$phi.start <- "default"
-        mcmc <- mcmc.control(S.scale = mcmc.input$S.scale,Htrunc=mcmc.input$Htrunc,S.start=mcmc.input$S.start,
-                             burn.in=mcmc.input$burn.in,thin=mcmc.input$thin,n.iter=mcmc.input$n.iter,
-                             phi.start=mcmc.input$phi.start,phi.scale=mcmc.input$phi.scale)
-      }
-    }
-  }
+  mcmc.input <- mcmc.check.aux(mcmc.input, fct="binom.krige.bayes")
   ##
   if(beta.prior == "fixed" | beta.prior == "normal") mean.d <- as.vector(trend.data %*% beta)
   else mean.d <- 0
@@ -472,230 +362,125 @@
   ##
   n.sim <- ncol(log.odds)
   if(inference) {
-    temp.post <- list()
-    temp.post$beta.mean <- array(NA, dim = c(beta.size, n.sim))
-    temp.post$beta.var <- array(NA, dim = c(beta.size, beta.size, n.sim))
-    temp.post$S2 <- rep(0, n.sim)
+    
+    ### dette afsnit skal skilles ud som en funktion paa et tidspunkt --------------
+    ##temp.post <- list()
+    ##temp.post$beta.mean <- array(NA, dim = c(beta.size, n.sim))
+    ##temp.post$beta.var <- array(NA, dim = c(beta.size, beta.size, n.sim))
+    ##temp.post$S2 <- rep(0, n.sim)
+    ##if(do.prediction) {
+    ##  temp.pred <- list()
+    ##  temp.pred$mean <- array(NA, dim = c(ni, n.sim))
+    ##  temp.pred$var <- array(NA, dim = c(ni, n.sim))
+    ##  if(output$sim.predict) {
+    ##    num.pred <- 1
+    ##    kb.results$predictive$simulations <- array(NA, dim = c(ni, n.sim))
+    ##  }
+    ##  else {
+    ##    num.pred <- 0
+    ##    kb.results$predictive$simulations <- " no simulations from the predictive distribution "
+    ##  }
+    ##}
+    ##else num.pred <- 0
+    ##model.temp <- model
+    ##model.temp$lambda <- 1
+    ##output.temp <- list(n.posterior = 0, n.predictive = num.pred, messages.screen = FALSE)
+    ##prior.temp <- prior
+    ##prior.temp$phi.prior <- "fixed"
+    ##prior.temp$phi.discrete <- NULL
+    ##prior.temp$priors.info <- NULL
+    ##if(phi.prior == "fixed" || length(phi.discrete) == 1) {
+    ##  if(phi.prior == "fixed") prior.temp$phi <- phi
+    ##  else prior.temp$phi <- phi.discrete
+    ##  temp.result <- krige.bayes.extnd(data = log.odds, coords = coords, locations = locations,
+    ##                                   model = model.temp, prior = prior.temp, output = output.temp)
+    ##  temp.post$beta.mean <- temp.result$posterior$beta$pars$mean
+    ##  temp.post$beta.var <- temp.result$posterior$beta$pars$var
+    ##  temp.post$S2 <- temp.result$posterior$sigmasq$pars$S2
+    ##  if(do.prediction) {
+    ##    temp.pred$mean <- temp.result$predictive$mean
+    ##    temp.pred$var <- temp.result$predictive$variance
+    ##    if(output$sim.predict)
+    ##      pred.simulations <- plogis(temp.result$predictive$simulations)
+    ##  }
+    ##}
+    ##else {
+    ##  len.phi.discrete <- length(phi.discrete)      
+    ##  step.phi.discrete <- phi.discrete[2] - phi.discrete[1]
+    ##  phi.table <- rep(0,len.phi.discrete)
+    ##  for(i in 1:len.phi.discrete){
+    ##    phi.table[i] <- sum(ifelse(abs(kb.results$posterior$phi$sample-phi.discrete[i])<0.5*step.phi.discrete,1,0))
+    ##  }
+    ##  phi.sample.unique <- phi.discrete[phi.table>0]
+    ##  phi.table <- phi.table[phi.table>0]
+    ##  len.phi.un <- length(phi.sample.unique)
+    ##  indic.phi <- array(rep(0, len.phi.un * max(phi.table)), dim = c(len.phi.un, max(phi.table)))
+    ##  numbers <- 1:length(kb.results$posterior$phi$sample)
+    ##  for(i in 1:len.phi.un) {
+    ##    temp.num <- numbers[abs(kb.results$posterior$phi$sample-phi.sample.unique[i])<0.5*step.phi.discrete]
+    ##    indic.phi[i, 1:length(temp.num)] <- temp.num
+    ##  }
+    ##  for(i in 1:len.phi.un){
+    ##    if(phi.table[i] == 1){
+    ##      prior.temp$phi <- phi.sample.unique[i]
+    ##      temp.result <- krige.bayes(data = log.odds[, indic.phi[i,1]], coords = coords, locations = locations, 
+    ##                                 model = model.temp, prior = prior.temp, output = output.temp)
+    ##      temp.post$beta.mean[, indic.phi[i,1]] <- temp.result$posterior$beta$pars$mean
+    ##      temp.post$beta.var[,  , indic.phi[i,1]] <- temp.result$posterior$beta$pars$var
+    ##      temp.post$S2[indic.phi[i,1]] <- temp.result$posterior$sigmasq$pars$S2
+    ##      if(do.prediction) {            
+    ##        temp.pred$mean[, indic.phi[i, 1]] <- temp.result$predictive$mean
+    ##        temp.pred$var[, indic.phi[i, 1]] <- temp.result$predictive$variance
+    ##        if(output$sim.predict) {
+    ##          pred.simulations[, indic.phi[i, 1]] <- plogis(temp.result$predictive$simulations)
+    ##        }
+    ##      }
+    ##    }
+    ##    else { 
+    ##      prior.temp$phi <- phi.sample.unique[i]
+    ##      temp.result <- krige.bayes.extnd(data = log.odds[, indic.phi[i,1:phi.table[i]]], coords = coords, locations = locations, 
+    ##                                      model = model.temp, prior = prior.temp, output = output.temp)  
+    ##      temp.post$beta.mean[, indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$mean         
+    ##      temp.post$beta.var[,  , indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$var
+    ##      temp.post$S2[indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$sigmasq$pars$S2       
+    ##      if(do.prediction) {
+    ##        temp.pred$mean[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$mean
+    ##        temp.pred$var[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$variance
+    ##        if(output$sim.predict) {
+    ##          pred.simulations[ , indic.phi[i, 1:phi.table[i]]] <- plogis(temp.result$predictive$simulations)
+    ##        }
+    ##      }
+    ##    }
+    ##  }
+    ##}
+    ##remove("temp.result")
+    ### ------- hertil
+
+    if(phi.prior=="fixed") phi.posterior <- list(phi.prior=phi.prior, phi=phi)
+    else  phi.posterior <- list(phi.prior=phi.prior, phi.discrete=phi.discrete, sample=kb.results$posterior$phi$sample)
+    predict.temp <- pred.aux(S=log.odds, coords=coords, locations=locations, model=model, prior=prior, output=output, phi.posterior=phi.posterior, link="logit")
+    temp.post <- predict.temp$temp.post
     if(do.prediction) {
-      temp.pred <- list()
-      temp.pred$mean <- array(NA, dim = c(ni, n.sim))
-      temp.pred$var <- array(NA, dim = c(ni, n.sim))
-      if(output$sim.predict) {
-        num.pred <- 1
-        kb.results$predictive$simulations <- array(NA, dim = c(ni, n.sim))
-      }
-      else {
-        num.pred <- 0
-        kb.results$predictive$simulations <- " no simulations from the predictive distribution "
-      }
+      temp.pred <- predict.temp$temp.pred
+      kb.results$predictive$simulations <- predict.temp$pred.simulations
     }
-    else num.pred <- 0
-    model.temp <- model
-    model.temp$lambda <- 1
-    output.temp <- list(n.posterior = 0, n.predictive = num.pred, messages.screen = FALSE)
-    prior.temp <- prior
-    prior.temp$phi.prior <- "fixed"
-    prior.temp$phi.discrete <- NULL
-    prior.temp$priors.info <- NULL
-    if(phi.prior == "fixed" || length(phi.discrete) == 1) {
-      if(phi.prior == "fixed") prior.temp$phi <- phi
-      else prior.temp$phi <- phi.discrete
-      temp.result <- krige.bayes.extnd(data = log.odds, coords = coords, locations = locations, 
-                                       model = model.temp, prior = prior.temp, output = output.temp)
-      temp.post$beta.mean <- temp.result$posterior$beta$pars$mean        
-      temp.post$beta.var <- temp.result$posterior$beta$pars$var
-      temp.post$S2 <- temp.result$posterior$sigmasq$pars$S2
-      if(do.prediction) {
-        temp.pred$mean <- temp.result$predictive$mean
-        temp.pred$var <- temp.result$predictive$variance
-        if(output$sim.predict)
-          kb.results$predictive$simulations <- plogis(temp.result$predictive$simulations)
-      }
-    }
-    else {
-      len.phi.discrete <- length(phi.discrete)      
-      step.phi.discrete <- phi.discrete[2] - phi.discrete[1]
-      phi.table <- rep(0,len.phi.discrete)
-      for(i in 1:len.phi.discrete){
-        phi.table[i] <- sum(ifelse(abs(kb.results$posterior$phi$sample-phi.discrete[i])<0.5*step.phi.discrete,1,0))
-      }
-      phi.sample.unique <- phi.discrete[phi.table>0]
-      phi.table <- phi.table[phi.table>0]
-      len.phi.un <- length(phi.sample.unique)
-      indic.phi <- array(rep(0, len.phi.un * max(phi.table)), dim = c(len.phi.un, max(phi.table)))
-      numbers <- 1:length(kb.results$posterior$phi$sample)
-      for(i in 1:len.phi.un) {
-        temp.num <- numbers[abs(kb.results$posterior$phi$sample-phi.sample.unique[i])<0.5*step.phi.discrete]
-        indic.phi[i, 1:length(temp.num)] <- temp.num
-      }
-      for(i in 1:len.phi.un){
-        if(phi.table[i] == 1){
-          prior.temp$phi <- phi.sample.unique[i]
-          temp.result <- krige.bayes(data = log.odds[, indic.phi[i,1]], coords = coords, locations = locations, 
-                                     model = model.temp, prior = prior.temp, output = output.temp)
-          temp.post$beta.mean[, indic.phi[i,1]] <- temp.result$posterior$beta$pars$mean
-          temp.post$beta.var[,  , indic.phi[i,1]] <- temp.result$posterior$beta$pars$var
-          temp.post$S2[indic.phi[i,1]] <- temp.result$posterior$sigmasq$pars$S2
-          if(do.prediction) {            
-            temp.pred$mean[, indic.phi[i, 1]] <- temp.result$predictive$mean
-            temp.pred$var[, indic.phi[i, 1]] <- temp.result$predictive$variance
-            if(output$sim.predict) {
-              kb.results$predictive$simulations[, indic.phi[i, 1]] <- plogis(temp.result$predictive$simulations)
-            }
-          }
-        }
-        else { 
-          prior.temp$phi <- phi.sample.unique[i]
-          temp.result <- krige.bayes.extnd(data = log.odds[, indic.phi[i,1:phi.table[i]]], coords = coords, locations = locations, 
-                                           model = model.temp, prior = prior.temp, output = output.temp)  
-          temp.post$beta.mean[, indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$mean         
-          temp.post$beta.var[,  , indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$beta$pars$var
-          temp.post$S2[indic.phi[i,1:phi.table[i]]] <- temp.result$posterior$sigmasq$pars$S2       
-          if(do.prediction) {
-            temp.pred$mean[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$mean
-            temp.pred$var[, indic.phi[i, 1:phi.table[i]]] <- temp.result$predictive$variance
-            if(output$sim.predict) {
-              kb.results$predictive$simulations[ , indic.phi[i, 1:phi.table[i]]] <- plogis(temp.result$predictive$simulations)
-            }
-          }
-        }
-      }
-    }
-    remove("temp.result")
+    ##
+    ##
     if(do.prediction) {
       ##
       d0mat <- loccoords(coords, locations)
-      loc.coincide <- (apply(d0mat < 1e-10, 2, sum) == 1)
+      loc.coincide <- (colSums(d0mat < 1e-10) == 1)
+
       if((is.logical(quantile.estimator) && (quantile.estimator)) || (is.numeric(quantile.estimator))){
-        ##
-        ##------- calculating medians and uncertainty
-        ##
-        temp.med <- apply(temp.pred$mean, 1, median)
-        temp.unc <- sqrt(apply(temp.pred$mean, 1, var) + apply(temp.pred$var, 1, median))
-        not.accurate <- (!loc.coincide)
-        diffe <- pmixed(temp.med, temp.pred,df.model)-0.5
-        temp.med.new <- temp.med[not.accurate]+0.1*(temp.med[not.accurate]+0.1) # to get started
-        inv.sl <- rep(0,ni)
-        parms.temp <- list()
-        while(any(not.accurate)){
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe.new <- pmixed(temp.med.new, parms.temp,df.model)-0.5
-          inv.sl[not.accurate] <- (temp.med.new-temp.med[not.accurate])/(diffe.new-diffe[not.accurate])
-          temp.med[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.med.new,temp.med[not.accurate])
-          diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
-          not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
-          temp.med.new <- temp.med[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
-        }
-        temp.upper <- qnorm(rep(0.975, ni), mean = temp.med, sd = temp.unc)
-        not.accurate <- (!loc.coincide)
-        diffe <- pmixed(temp.upper, temp.pred,df.model)-0.975
-        temp.upper.new <- temp.upper[not.accurate]+0.5*(temp.upper[not.accurate]+0.5) # to get started
-        inv.sl <- rep(0,ni)      
-        while(any(not.accurate)){
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe.new <- pmixed(temp.upper.new, parms.temp,df.model)-0.975
-          inv.sl[not.accurate] <- (temp.upper.new-temp.upper[not.accurate])/(diffe.new-diffe[not.accurate])
-          temp.upper[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.upper.new,temp.upper[not.accurate])
-          diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
-          not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
-          temp.upper.new <- temp.upper[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
-        }     
-        temp.lower <- qnorm(rep(0.025, ni), mean = temp.med, sd = temp.unc)
-        not.accurate <- (!loc.coincide)
-        diffe <- pmixed(temp.lower, temp.pred,df.model)-0.025
-        temp.lower.new <- temp.lower[not.accurate]+0.5*(temp.lower[not.accurate]+0.5) # to get started
-        inv.sl <- rep(0,ni)
-        while(any(not.accurate)){
-          parms.temp$mean<-temp.pred$mean[not.accurate,,drop=FALSE]
-          parms.temp$var<-temp.pred$var[not.accurate,,drop=FALSE]
-          diffe.new <- pmixed(temp.lower.new,parms.temp,df.model)-0.025
-          inv.sl[not.accurate] <- (temp.lower.new-temp.lower[not.accurate])/(diffe.new-diffe[not.accurate])
-          temp.lower[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.lower.new,temp.lower[not.accurate])
-          diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
-          not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
-          temp.lower.new <- temp.lower[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
-        }
-        if(any(loc.coincide)){
-          temp.med[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, median)
-          temp.upper[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = 0.975)
-          temp.lower[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = 0.025) 
-        }
-        kb.results$predictive$median <- plogis(temp.med)
-        kb.results$predictive$uncertainty <- (plogis(temp.upper) - plogis(temp.lower))/4
-        ## calculating quantiles
-        if(is.logical(quantile.estimator) && (quantile.estimator)){
-          kb.results$predictive$quantiles <- as.data.frame(cbind(plogis(temp.lower), plogis(temp.med), plogis(temp.upper)))
-        }
-        if(is.numeric(quantile.estimator)) {
-          nmq <- length(quantile.estimator)
-          if(nmq > 1) {
-            temp.quan <- matrix(NA, ni, nmq)
-            dig <- rep(3, nmq)
-            for(i in 1:nmq) {
-              while(quantile.estimator[i] != round(quantile.estimator[i], digits = dig[i])) dig[i] <-dig[i] + 1
-              temp.quan[, i] <- qnorm(rep(quantile.estimator[i], ni), mean = temp.med, sd = temp.unc)
-              if(any(loc.coincide)) temp.quan[loc.coincide, i] <- temp.med[loc.coincide]
-              not.accurate <- (!loc.coincide)
-              diffe <- pmixed(temp.quan[,i], temp.pred,df.model)-quantile.estimator[i]
-              numb <- 0.1+abs(quantile.estimator[i]-0.5)
-              temp.quan.new <- temp.quan[not.accurate,i]+numb*(temp.quan[not.accurate,i]+numb) # to get started
-              inv.sl <- rep(0,ni)
-              while(any(not.accurate)) {
-                parms.temp$mean <-temp.pred$mean[not.accurate,,drop=FALSE]
-                parms.temp$var <-temp.pred$var[not.accurate,,drop=FALSE]
-                diffe.new <- pmixed(temp.quan.new,parms.temp,df.model)-quantile.estimator[i]
-                inv.sl[not.accurate] <- (temp.quan.new-temp.quan[not.accurate, i])/(diffe.new-diffe[not.accurate])
-                temp.quan[not.accurate, i] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.quan.new,temp.quan[not.accurate, i])
-                diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
-                not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
-                temp.quan.new <- temp.quan[not.accurate, i] - diffe[not.accurate]*inv.sl[not.accurate]
-              }
-              if(any(loc.coincide)){
-                temp.quan[loc.coincide,i] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = quantile.estimator[i])
-              }
-            }
-            kb.results$predictive$quantiles <- as.data.frame(plogis(temp.quan))
-          }
-          else {
-            dig <- 3
-            while(quantile.estimator != round(quantile.estimator,digits = dig)) dig <- dig + 1
-            temp.quan <- qnorm(rep(quantile.estimator,ni), mean = temp.med, sd = temp.unc)
-            not.accurate <- (!loc.coincide)
-            diffe <- pmixed(temp.quan, temp.pred,df.model)-quantile.estimator
-            numb <- 0.1+abs(quantile.estimator-0.5)
-            temp.quan.new <- temp.quan[not.accurate]+numb*(temp.quan[not.accurate]+numb) # to get started
-            inv.sl <- rep(0,ni)
-            while(any(not.accurate)) {
-              parms.temp$mean <-temp.pred$mean[not.accurate,,drop=FALSE]
-              parms.temp$var <-temp.pred$var[not.accurate,,drop=FALSE]
-              diffe.new <- pmixed(temp.quan.new,parms.temp,df.model)-quantile.estimator
-              inv.sl[not.accurate] <- (temp.quan.new-temp.quan[not.accurate])/(diffe.new-diffe[not.accurate])
-              temp.quan[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), temp.quan.new,temp.quan[not.accurate])
-              diffe[not.accurate] <- ifelse(abs(diffe[not.accurate]) > abs(diffe.new), diffe.new, diffe[not.accurate])
-              not.accurate[not.accurate] <- ifelse(abs(diffe[not.accurate])>0.0005, TRUE, FALSE)
-              temp.quan.new <- temp.quan[not.accurate] - diffe[not.accurate]*inv.sl[not.accurate]
-            }
-            if(any(loc.coincide)){
-              temp.quan[loc.coincide] <- apply(temp.pred$mean[loc.coincide,,drop=FALSE], 1, quantile, probs = quantile.estimator)
-            }
-            kb.results$predictive$quantiles <- as.vector(plogis(temp.quan))
-          }
-        }
-        if(is.logical(quantile.estimator) && (quantile.estimator)){
-          qname <- rep(0, 3)
-          qname[1] <- paste("q0.025", sep = "")
-          qname[2] <- paste("q0.5", sep = "")
-          qname[3] <- paste("q0.975", sep = "")
-          names(kb.results$predictive$quantiles) <- qname
-        }
-        if(is.numeric(quantile.estimator) && nmq > 1) {
-          qname <- rep(0, length(quantile.estimator))
-          for(i in 1:length(quantile.estimator))
-            qname[i] <- paste("q", 100 * quantile.estimator[i], sep = "")
-          names(kb.results$predictive$quantiles) <- qname
+        predi.q <- pred.quan.aux(temp.pred, loc.coincide, df.model, ni, quantile.estimator)
+        kb.results$predictive$median <- plogis(predi.q$median)
+        kb.results$predictive$uncertainty <- (plogis(predi.q$upper) - plogis(predi.q$lower))/4      
+        if(is.data.frame(predi.q$quantiles)){
+          names.q <- names(predi.q$quantiles)
+          kb.results$predictive$quantiles <- as.data.frame(plogis(as.matrix(predi.q$quantiles)))
+          names(kb.results$predictive$quantiles) <- names.q
         }
+        else kb.results$predictive$quantiles <- plogis(predi.q$quantiles)
       }
       ##
       ## ------ probability estimators
@@ -729,9 +514,9 @@
     if(beta.prior == "fixed") kb.results$posterior$beta <- paste("provided by user: ", beta)
     else {
       kb.results$posterior$beta <- list()
-      kb.results$posterior$beta$mean <- apply(temp.post$beta.mean, 1, mean)
+      kb.results$posterior$beta$mean <- rowMeans(temp.post$beta.mean)
       names(kb.results$posterior$beta$mean) <- beta.names
-      kb.results$posterior$beta$var <- apply(temp.post$beta.var, c(1, 2), mean) + var(t(temp.post$beta.mean))
+      kb.results$posterior$beta$var <- rowMeans(temp.post$beta.var, dims = 2) + var(t(temp.post$beta.mean))
       dimnames(kb.results$posterior$beta$var) <- list(beta.names,beta.names)
     }
     if(sigmasq.prior == "fixed") kb.results$posterior$sigmasq <- paste("provided by user: ", sigmasq) 
@@ -777,8 +562,7 @@
         }
       }
     }
-    if(is.R()) remove("temp.post")
-    else remove("temp.post", frame = sys.nframe())
+    remove("temp.post")
   }
   if(output$keep.mcmc.sim) kb.results$posterior$simulations <- plogis(log.odds)  
   model$lambda <- NULL
@@ -787,8 +571,9 @@
   kb.results$mcmc.input <- mcmc.input
   kb.results$.Random.seed <- seed
   kb.results$call <- call.fc
-  ##attr(kb.results, "class") <- c("krige.bayes", "kriging")
-  attr(kb.results, "class") <- c("binom.krige.bayes")
+  attr(kb.results, "prediction.locations") <- call.fc$locations
+  if(!is.null(call.fc$borders)) attr(kb.results, "borders") <- call.fc$borders
+  attr(kb.results, "class") <- "glm.krige.bayes"
   if(messages.screen) cat("binom.krige.bayes: done!\n")
   return(kb.results)
 }
