@@ -60,8 +60,8 @@
       return(result)
     }
   result <- array(unlist(lapply(as.data.frame(data1), bin.f, temp.list = temp.list)), dim = c(nbins, 2, n.datasets))
-  mm2 <- apply(as.matrix(data1), 2, mean)^2
-  sigma2 <- log(t(t((apply(as.matrix(data1), 2, var) * (n.data - 1))/n.data - apply(as.matrix(data1/units.m), 2, mean))/mm2) + 1)
+  mm2 <- colMeans(as.matrix(data1))^2
+  sigma2 <- log(t(t((apply(as.matrix(data1), 2, var) * (n.data - 1))/n.data - colMeans(as.matrix(data1/units.m)))/mm2) + 1)
   if(estimator.type == "poisson")
     calcresult <- array(rbind(as.vector(sigma2), log(t(t(as.matrix(result[, 1,  ]))/mm2))), dim = c(nbins + 1, n.datasets))
   else calcresult <- array(log(t(t(as.matrix(result[, 1,  ]))/mm2)), dim = c(nbins, n.datasets))
@@ -140,9 +140,8 @@ function(x, max.dist = x$max.dist, ...)
   if (is.null(max.dist)) stop("argument max.dist needed for this object")
   my.l <- x
   if(is.null(my.l$nugget)) my.l$nugget <- 0
-  C.f <- function(x, my.l){
-    if(x>0) return(cov.spatial(x, cov.model = my.l$cov.model, kappa = my.l$kappa, cov.pars = my.l$cov.pars))
-    else return(my.l$nugget+cov.spatial(x, cov.model = my.l$cov.model, kappa = my.l$kappa, cov.pars = my.l$cov.pars))
+  C.f <- function(x, my.l){ 
+    return(ifelse(x>0, 0, my.l$nugget)+cov.spatial(x, cov.model = my.l$cov.model, kappa = my.l$kappa, cov.pars = my.l$cov.pars))
   }
   curve(C.f(x,my.l=my.l), from = 0, to = max.dist, add=TRUE, ...)
   return(invisible())
