@@ -11,7 +11,8 @@
   cat(package.description("geoRglm", lib = lib, field="Title"))
   cat("\n")
   ver <- package.description("geoRglm", lib = lib, field="Version")
-  cat(paste("geoRglm version", ver,  "is now loaded\n"))
+  dat <- package.description("geoRglm", lib = lib, field="Date")
+  cat(paste("geoRglm version ", ver, " (", dat, ") is now loaded\n", sep=""))
   cat("-----------------------------------------------------------\n")
   cat("\n")
   return(invisible(0))
